c     Created by Mahdi Esmaily Moghadam 05-25-2011
c     Please report any problem to mesmaily@ucsd.edu, memt63@yahoo.com

c Here data are received from Phasta and it setup the data required for
c integration of ODE's inside FINDX
      PROGRAM GenBC

      USE COM

      IMPLICIT NONE

      INTEGER i, n, nTimeStep

      REAL(8) t, dt, tFinal, pMin, temp

      CHARACTER(8048) string
      CHARACTER(32) sTmp
      CHARACTER flag

      REAL(8), ALLOCATABLE :: Qi(:), Qf(:), Pi(:), Pf(:), X(:), Xo(:),
     2   f(:,:)

c     flag = I : Initializing
c     flag = T : Iteration loop
c     flag = L : Last iteration
c     flag = D : Derivative
c
c   Here is the period of time that you should integrate for
c   each of these flags:
c
c Flags              I                                                D&T&L
c                    ^                                                  ^
c 3D code time step: N.................................................N+1
c 0D code time step: 1..............................................nTimeStep+1
c Flowrates:         Qi.................................................Qf
c Time, t:         tInitial......................................tInitial+tFinal


      CALL INITIALIZE(nTimeStep)

c********************************************************************
c Block for reading the data from phasta
      OPEN (1, FILE='GenBC.int', STATUS='OLD', FORM='UNFORMATTED');
      READ (1) flag
      READ (1) tFinal
      READ (1) i
      IF (i .NE. nDirichletSrfs) THEN
         PRINT *, 'ERROR: Number of Dirichlet Surfaces from Phasta is:',
     2      i
         PRINT *, 'While nDirichletSrfs is equal to:', nDirichletSrfs
         PRINT *
         PRINT *, 'Number of Dirichlet surfaces defined in solver.inp',
     2      ' should match with nDirichletSrfs defined in USER.f'
         STOP
      END IF
      READ (1) i
      IF (i .NE. nNeumannSrfs) THEN
         PRINT *, 'ERROR: Number of Neumann Surfaces from Phasta is:',
     2      i
         PRINT *, 'While nNeumannSrfs is equal to:', nNeumannSrfs
         PRINT *
         PRINT *, 'Number of Neumann surfaces defined in solver.inp',
     2      ' should match with nNeumannSrfs defined in USER.f'
         STOP
      END IF
      IF (nDirichletSrfs .GT. 0) THEN
         IF (qCorr .OR. pCorr) THEN
            PRINT *, 'ERROR: You should only use P/Q correction when',
     2         ' all the surfaces are Neumann surfaces'
            STOP
         END IF
      END IF
      DO i=1, nDirichletSrfs
         IF (srfToXdPtr(i).LT.1 .OR. srfToXdPtr(i).GT.nUnknowns) THEN
            PRINT *, "ERROR: srfToXdPtr(", i, ") is unexpected"
            STOP
         END IF
      END DO
      DO i=1, nNeumannSrfs
         IF (srfToXPtr(i).LT.1 .OR. srfToXPtr(i).GT.nUnknowns) THEN
            PRINT *, "ERROR: srfToXPtr(", i, ") is unexpected"
            STOP
         END IF
      END DO

      ALLOCATE (Pi(nDirichletSrfs), Pf(nDirichletSrfs),
     2   PDirichlet(nDirichletSrfs,4))
      ALLOCATE (Qi(nNeumannSrfs), Qf(nNeumannSrfs),
     2   QNeumann(nNeumannSrfs,4))

      DO i=1, nDirichletSrfs
         READ(1) Pi(i), Pf(i)
      END DO
      Pi = Pi/pConv
      Pf = Pf/pConv

      DO i=1,nNeumannSrfs
         READ (1) Qi(i), Qf(i)
      END DO
      CLOSE(1)
      Qi = Qi/qConv
      Qf = Qf/qConv

c********************************************************************
c Block for initializing the unknowns

      IF (flag .EQ. 'I') THEN
         nTimeStep = 0
      ELSE
         dt = tFinal/REAL(nTimeStep,8)
      END IF
      ALLOCATE (X(nUnknowns), Xo(nUnknowns), f(nUnknowns,4),
     2   offset(nUnknowns), Xprint(nXprint))

      OPEN (1, FILE='InitialData', STATUS='OLD', FORM='UNFORMATTED')
      READ (1) t

      DO i=1,nUnknowns
          READ (1) Xo(i)
      END DO
      CLOSE (1)

c********************************************************************
c Setting up the system of equations
      offset = 0D0
      DO n=1, nTimeStep
         DO i=1, 4
            temp = (REAL(n-1,8) + REAL(i-1,8)/3D0)/REAL(nTimeStep,8)

            QNeumann(:,i)   = Qi + (Qf-Qi)*temp
            PDirichlet(:,i) = Pi + (Pf-Pi)*temp

            IF (qCorr) THEN
               temp = SUM(QNeumann(:,i))/REAL(nNeumannSrfs,8)
               QNeumann(:,i) = QNeumann(:,i) - temp
            END IF
         END DO

         CALL FINDF (t, Xo, f(:,1), QNeumann(:,1),
     2      PDirichlet(:,1))
         X = Xo + dt*f(:,1)/3D0


         CALL FINDF (t+dt/3D0, X, f(:,2), QNeumann(:,2),
     2      PDirichlet(:,2))
         X = Xo - dt*f(:,1)/3D0 + dt*f(:,2)

         CALL FINDF (t+dt*2D0/3D0, X, f(:,3), QNeumann(:,3),
     2      PDirichlet(:,3))
         X = Xo + dt*f(:,1) - dt*f(:,2) + dt*f(:,3)

         CALL FINDF (t+dt, X, f(:,4), QNeumann(:,4),
     2      PDirichlet(:,4))

         f(:,1) = (f(:,1) + 3D0*f(:,2) + 3D0*f(:,3) + f(:,4))/8D0

         Xo = Xo + dt*f(:,1)
         t = t + dt
      END DO

c********************************************************************
c Time to write the results

      X = Xo
      IF (pCorr .AND. flag.NE.'D') THEN
         pMin = X(srfToXPtr(1)) + offset(srfToXPtr(1))
         DO i=2, nNeumannSrfs
            IF (X(srfToXPtr(i)) .LT. pMin) THEN
               pMin = X(srfToXPtr(i)) + offset(srfToXPtr(i))
            END IF
         END DO
      ELSE
         pMin = 0D0
      END IF

c Writing nDirichlet flowrates here
      OPEN (1, FILE='GenBC.int', STATUS='OLD', FORM='UNFORMATTED')
      DO i=1, nDirichletSrfs
         WRITE (1) (X(srfToXdPtr(i)) + offset(srfToXdPtr(i)))*qConv
c         WRITE (*,*) (X(srfToXdPtr(i)))
      END DO

c Writing nNeumannSrfs pressures here
      DO i=1, nNeumannSrfs
         WRITE (1) (X(srfToXPtr(i)) - pMin
     2      + offset(srfToXPtr(i)))*pConv
      END DO
      CLOSE(1)

      IF (flag .EQ. 'L') THEN
         OPEN(1, FILE='InitialData', STATUS='OLD', FORM='UNFORMATTED');
         WRITE (1) t
         DO i=1,nUnknowns
            WRITE (1) X(i)
         END DO
         CLOSE(1)

         OPEN(1, FILE='AllData', STATUS='UNKNOWN', ACCESS='APPEND');
         string = ''
         DO i=1, nUnknowns
            WRITE (sTmp,"(ES14.6E2)") X(i)
            string = TRIM(string)//sTmp
         END DO
         DO i=1, nXprint
            WRITE (sTmp,"(ES14.6E2)") Xprint(i)
            string = TRIM(string)//sTmp
         END DO
         WRITE (1,"(A)") TRIM(string)
         CLOSE(1)
      END IF

      STOP
      END PROGRAM GenBC
