c     USER.F framework was created by Mahdi Esmaily Moghadam 5-23-2011
c     Please report any problem to mesmaily@ucsd.edu, memt63@yahoo.com
c
c <^>
c-----
c(~ ~)
c /(`
c   )
c--------------------------------------------------------------------
      SUBROUTINE INITIALIZE(nTimeStep)

      USE COM

      IMPLICIT NONE

      LOGICAl ierr

      INTEGER nTimeStep
      INTEGER i

      REAL(8), ALLOCATABLE :: tZeroX(:)

      INTENT(OUT) nTimeStep

c********************************************************************
c For instance if pressure in 3D solver is in cgs and here mmHg
c pConv=1334=1.334D3, also if flowrate in 3D solver is mm^3/s and
c here is mL/s qConv=1000=1D3. In the case both solver are using same
c unites you can set these two conversion coefficients to 1D0
      pConv = 1D0
      qConv = 1D0

c Current USER.f has values in clinical notation : mmHg
c Conversion : pClin=pCGS/pConv; QrClinical=QrCGS;RClinin=RCGS/pConv;
c Conversion : Lclinic=LCGS/pConv;CClinic=CCGS/pConv;

c Only when all the surfaces of you model are coupled with NeumannSrfs
c you may set this to .TRUE.
      pCorr = .FALSE.
      qCorr = .FALSE.

c********************************************************************
c Block for your inputs

c These two value should match to that of defined in solver.inp
      nDirichletSrfs = 0
      nNeumannSrfs   = 8
c Number of unknowns that you need inside your lumped parameter network
c For coronaries, we have 1 inlet pressure + 3 * noutlets, where 3 is the
c outlet pressure, pressure drop across Ca and pressure drop across Cim (in order)
      nUnknowns      = 22 ! (1 + 3 * 7)
c Number of time step between N and N+alpha, for higher values this code
c would be more accurate and more costly
      nTimeStep = 100

c Number of parameters to be printed in AllData file (the first
c nUnknowns columns are by default X)
      nXprint = 3

c--------------------------------------------------------------------
c You don't need to change this part                                 !
      ALLOCATE (tZeroX(nUnknowns), srfToXdPtr(nDirichletSrfs))       !
      ALLOCATE (srfToXPtr(nNeumannSrfs))                             !
      tZeroX = 0D0                                                   !
c--------------------------------------------------------------------

c Value of your unknown at time equal to zero (This is going to be used
c ONLY when you are initiating simulation)

C INITIALIZE INLET FLOW AND RCR CAPACITOR PRESSURE
c      tZeroX = (/0D0, 0D0, 0D0, 0D0, 0D0, 0D0, 0D0, 0D0, /)

c--------------------------------------------------------------------
c You don't need to change this part                                 !
      INQUIRE (FILE='InitialData', EXIST=ierr)                       !
      IF (.NOT.ierr) THEN                                            !
         PRINT *, 'Initializing unknowns in LPM'                     !
         OPEN (1, FILE='InitialData',STATUS='NEW',FORM='UNFORMATTED')!
         WRITE (1) 0D0                                               !
         DO i=1, nUnknowns                                           !
            WRITE (1) tZeroX(i)                                      !
         END DO                                                      !
         CLOSE(1)                                                    !
      END IF                                                         !
c--------------------------------------------------------------------

c Surface to X pointer: this defines which Unknown corresponds to which
c suface in "List of Neumann Surfaces" inside solver.inp
c For example, if you have "List of Neumann Surfaces= 2 8 4 ...."
c and you set "srfToXPtr = (/5,3,9,.../)"
C this means X(5) corresponds to surface 2, X(3) <-> surface 8,
c and X(9) <-> surface 4
c      srfToXPtr = (/7/)
c This is exactly the same for srfToXdPtr only srfToXdPtr is for Dirichlet
c surfaces
      srfToXPtr = (/1, 2, 5, 8, 11, 14, 17, 20/)
c                  RCA RCA_ c1 c2 c3 c4 c5 c6

c--------------------------------------------------------------------

c We open the inlet pressure file.
      CALL COUNTILINES(1, '../inlet_pressure.txt', nInletPressureSteps)
      ALLOCATE(timesInletPressure(nInletPressureSteps),
     &         inletPressures(nInletPressureSteps))
      CALL READIFILE(2, '../inlet_pressure.txt',
     &               timesInletPressure, inletPressures,
     &               nInletPressureSteps)

c We open the myocardial pressure file
      CALL COUNTILINES(3, '../distal_pressure.txt', nPimSteps)
      ALLOCATE(timesPim(nPimSteps),
     &         Pim(nPimSteps))
      CALL READIFILE(4, '../distal_pressure.txt',
     &               timesPim, Pim, nPimSteps)

      END SUBROUTINE INITIALIZE

c####################################################################
c Here you should find the f_i=dx_i/dt, based on the following parameters:
c  current x_i:                   x(i)
c  Current time:                  t
c  Flowrates of Neumann faces:    Q(i)
c  Pressures of Dirichlet faces:  P(i)

      SUBROUTINE FINDF(t, x, f, Q, P)

      USE COM

      IMPLICIT NONE

      REAL*8 t,x(nUnknowns), f(nUnknowns), Q(nNeumannSrfs),
     2   P(nDirichletSrfs),tau,tt
      INTEGER i

      INTENT(IN) t, Q
      INTENT(INOUT) x
      INTENT(OUT) f

      REAL(8) :: cPressure
      REAL(8) :: cPim

c Resistances and capacitances for each outlet
      REAL(8) :: Ra_c1 = 189641.981968868
      REAL(8) :: Ca_c1 = 2.720742116120772e-07
      REAL(8) :: Rmicro_c1 = 308168.22069941054
      REAL(8) :: Cim_c1 = 2.201327712134079e-06
      REAL(8) :: Rv_c1 = 94820.990984434

      REAL(8) :: Ra_c2 = 149816.76376986617
      REAL(8) :: Ca_c2 = 3.2616448254652425e-07
      REAL(8) :: Rmicro_c2 = 243452.24112603255
      REAL(8) :: Cim_c2 = 2.6389671769673324e-06
      REAL(8) :: Rv_c2 = 74908.38188493309

      REAL(8) :: Ra_c3 = 289552.34876668686
      REAL(8) :: Ca_c3 = 1.9647552196776434e-07
      REAL(8) :: Rmicro_c3 = 470522.56674586615
      REAL(8) :: Cim_c3 = 1.5896655868300932e-06
      REAL(8) :: Rv_c3 = 144776.17438334343

      REAL(8) :: Ra_c6 = 245587.47305448688
      REAL(8) :: Ca_c6 = 2.2301009054332464e-07
      REAL(8) :: Rmicro_c6 = 399079.6437135412
      REAL(8) :: Cim_c6 = 1.8043543689414448e-06
      REAL(8) :: Rv_c6 = 122793.73652724344

      REAL(8) :: Ra_c4 = 243770.88258185025
      REAL(8) :: Ca_c4 = 2.242873640760869e-07
      REAL(8) :: Rmicro_c4 = 396127.6841955067
      REAL(8) :: Cim_c4 = 1.8146886729792486e-06
      REAL(8) :: Rv_c4 = 121885.44129092513

      REAL(8) :: Ra_c5 = 315181.2732436974
      REAL(8) :: Ca_c5 = 1.8406666055834828e-07
      REAL(8) :: Rmicro_c5 = 512169.5690210083
      REAL(8) :: Cim_c5 = 1.489266617244818e-06
      REAL(8) :: Rv_c5 = 157590.6366218487

      REAL(8) :: Ra_RCA = 244288.55481398207
      REAL(8) :: Ca_RCA = 2.2392166869587423e-07
      REAL(8) :: Rmicro_RCA = 396968.90157272085
      REAL(8) :: Cim_RCA = 1.8117298649029825e-06
      REAL(8) :: Rv_RCA = 122144.27740699104

c Calculate current value of inlet pressure and distal pressure through linear
c interpolation
      tt=dmod(t,timesInletPressure(nInletPressureSteps)) ! we retrieve a periodic tt
      DO i=1,nInletPressureSteps-1
        IF (tt .GE. timesInletPressure(i) .AND.
     &      tt .LE. timesInletPressure(i+1) ) THEN
            cPressure=inletPressures(i)+
     &  (inletPressures(i+1)-inletPressures(i))/
     &  (timesInletPressure(i+1)-timesInletPressure(i))*
     &  (tt-timesInletPressure(i))
            EXIT
        ENDIF
      ENDDO

      tt=dmod(t,timesPim(nPimSteps)) ! we retrieve a periodic tt
      DO i=1,nPimSteps-1
        IF (tt .GE. timesPim(i) .AND.
     &      tt .LE. timesPim(i+1) ) THEN
            cPim=Pim(i)+
     &   (Pim(i+1)-Pim(i))/
     &   (timesPim(i+1)-timesPim(i))*
     &   (tt-timesPim(i))
            EXIT
         ENDIF
      ENDDO

c Initialize f at zero
      f = 0
      x(1) = cPressure

c Equation for RCA (outlet)
      i = 2
      x(i) = Ra_RCA * Q(2) + x(i+1)
      f(i) = 0.0
      f(i+1) = Q(2) / Ca_RCA -
     &       (x(i+1) - x(i+2) - cPim) / (Rmicro_RCA * Ca_RCA)
      f(i+2) = (x(i+1) - x(i+2) -cPim) / (Rmicro_RCA * Cim_RCA) -
     &        (x(i+2) + cPim) / (Rv_RCA * Cim_RCA)
c

c Equation for c1
      i = 5
      x(i) = Ra_c1 * Q(3) + x(i+1)
      f(i) = 0.0
      f(i+1) = Q(3) / Ca_c1 -
     &       (x(i+1) - x(i+2) - cPim) / (Rmicro_c1 * Ca_c1)
      f(i+2) = (x(i+1) - x(i+2) - cPim) / (Rmicro_c1 * Cim_c1) -
     &        (x(i+2) + cPim) / (Rv_c1 * Cim_c1)
c

c Equation for c2
      i = 8
      x(i) = Ra_c2 * Q(4) + x(i+1)
      f(i) = 0.0
      f(i+1) = Q(4) / Ca_c2 -
     &       (x(i+1) - x(i+2) - cPim) / (Rmicro_c2 * Ca_c2)
      f(i+2) = (x(i+1) - x(i+2) - cPim) / (Rmicro_c2 * Cim_c2) -
     &        (x(i+2) + cPim) / (Rv_c2 * Cim_c2)
c

c Equation for c3
      i = 11
      x(i) = Ra_c3 * Q(5) + x(i+1)
      f(i) = 0.0
      f(i+1) = Q(5) / Ca_c3 -
     &       (x(i+1) - x(i+2) - cPim) / (Rmicro_c3 * Ca_c3)
      f(i+2) = (x(i+1) - x(i+2) - cPim) / (Rmicro_c3 * Cim_c3) -
     &        (x(i+2) + cPim) / (Rv_c3 * Cim_c3)
c

c Equation for c4
      i = 14
      x(i) = Ra_c4 * Q(6) + x(i+1)
      f(i) = 0.0
      f(i+1) = Q(6) / Ca_c4 -
     &       (x(i+1) - x(i+2) - cPim) / (Rmicro_c4 * Ca_c4)
      f(i+2) = (x(i+1) - x(i+2) - cPim) / (Rmicro_c4 * Cim_c4) -
     &        (x(i+2) + cPim) / (Rv_c4 * Cim_c4)
c

c Equation for c5
      i = 17
      x(i) = Ra_c5 * Q(7) + x(i+1)
      f(i) = 0.0
      f(i+1) = Q(7) / Ca_c5 -
     &       (x(i+1) - x(i+2) - cPim) / (Rmicro_c5 * Ca_c5)
      f(i+2) = (x(i+1) - x(i+2) - cPim) / (Rmicro_c5 * Cim_c5) -
     &        (x(i+2) + cPim) / (Rv_c5 * Cim_c5)
c

c Equation for c6
      i = 20
      x(i) = Ra_c6 * Q(8) + x(i+1)
      f(i) = 0.0
      f(i+1) = Q(8) / Ca_c6 -
     &       (x(i+1) - x(i+2) - cPim) / (Rmicro_c6 * Ca_c6)
      f(i+2) = (x(i+1) - x(i+2) - cPim) / (Rmicro_c6 * Cim_c6) -
     &        (x(i+2) + cPim) / (Rv_c6 * Cim_c6)
c

c Print additional data into AllData, unknowns X(n) are always written into the first n columns of AllData then followed by additional variables Xprint defined by users.
      Xprint(1) = t
      Xprint(2) = x(1)
      Xprint(3) = Q(1)
      RETURN

      END SUBROUTINE FINDF
